# A VMD script that makes index files for sorting/unsorting
# residues in GROMACS files
# Jerome Henin <jhenin@cmm.upenn.edu>, July 2006
#
# You need to load two files into VMD:
# Frame 0: system, unsorted
# Frame 1: same coordinates, but sorted residues
# Produces an index file, sort.ndx, to convert 0 into 1
#
# Typical usage:
# vmd -dispdev text unsorted.gro sorted.gro -e sort.tcl

proc unsort { indexFileName tol } {

    set all [atomselect top all frame 0]
    set residues [lsort -integer -unique [$all get residue]]

    set out [open $indexFileName w] 
    
    puts $out "\[ Unsort \]"

    foreach r $residues {
	if {[expr {$r % 1000 == 0}]} {
	    puts "Treating residue $r..."
	}

	set sel [atomselect top "residue $r" frame 1]

	foreach {x y z} [lindex [$sel get {x y z}] 0] {}

	set xmin [expr {$x-$tol}]
	set xmax [expr {$x+$tol}]
	set ymin [expr {$y-$tol}]
	set ymax [expr {$y+$tol}]
	set zmin [expr {$z-$tol}]
	set zmax [expr {$z+$tol}]

	set target [atomselect top "x>$xmin and x<$xmax and y>$ymin and y<$ymax and z>$zmin and z<$zmax" frame 0]

	if {[$target num] != 1} {
	    puts "ERROR: first atom in residue $r not found within $tol Angstroms in target system"
	    exit		
	}

	set tresidue [lindex [$target get residue] 0]
	set targetRes [atomselect top "residue $tresidue"]

	# Beware 0-based VMD indices !!!
	foreach index [$targetRes list] {
	    puts -nonewline $out "[expr {$index + 1}] "
	}
	# One residue per line
	puts $out ""

	$targetRes delete	
	$sel delete
	$target delete
    }

    close $out
    puts "Done writing file $indexFileName."
}

# Run the procedure

# Default .1 Angstrom tolerance for recognizing atoms
# Increase if you only have slightly different configs
# but this also increases the risk for errors...

unsort sort.ndx 0.1

exit
